	'
	' Atomchess for Intellivision
	'
	' (c) Copyright 2023 Oscar Toledo G.
	' https://nanochess.org/
	'
	' Creation date: Feb/28/2023.
	'

	CONST DEBOUNCE_TIME = 15

	' TODO:
	' * Debug.

	CONST STACK_ENTRY = 10	' This should be SYNCHRONIZED with the chess engine

	DIM board(120)

	IF NTSC THEN frames = 60 ELSE frames = 50

	level = 1

	CLS
	MODE 1
	GOSUB clear_sprites
	WAIT
	DEFINE 0,14,pieces_bitmaps_0
	WAIT

	GOSUB setup_board
	turn = 0
	cursor = 64
	selected = 84
	debounce = DEBOUNCE_TIME

	' Variables used by the engine assembler code
	total_movements = 0
	en_passant = 0
	best_score = 0
	best_origin = 0
	best_target = 0

	#stack_base = VARPTR board(120)
	#stack = #stack_base
	#depth = #stack_base

	GOSUB update_board

main_loop:
	c = square_to_screen(cursor)
	x = c % 20 * 8
	y = c / 20 * 8
	IF FRAME AND 4 THEN
		SPRITE 0,$0308+x,$0108+y,$1802 + 12 * 8
		SPRITE 1,$0310+x,$0108+y,$180a + 12 * 8
	ELSE
		SPRITE 0,0
		SPRITE 1,0
	END IF

	WAIT

	c = CONT
	IF debounce THEN c = 0: debounce = debounce - 1
	d = c AND $e0
	IF (d = $80) + (d = $40) + (d = $20) THEN	' Keypad
	ELSE
		IF (d = $a0) + (d = $c0) + (d = $60) THEN	' Button
			SPRITE 0, 0
			SPRITE 1, 0

			valid = 0
			c = board(cursor) AND $0F
			IF turn <> 0 THEN
				IF c >= $01 AND c <= $06 THEN valid = 1
				IF c = 0 OR (c >= $09 AND c <= $0e) THEN valid = valid + 2
			ELSE
				IF c >= $09 AND c <= $0e THEN valid = 1
				IF c = 0 OR (c >= $01 AND c <= $06) THEN valid = valid + 2
			END IF

			IF valid AND 1 then
				GOSUB deselect_square
				c = cursor
				GOSUB select_square
				debounce = DEBOUNCE_TIME
				c = 0
			ELSEIF (selected <> 0) AND (valid AND 2) <> 0 THEN

				origin = selected
				target = cursor
				selected = 0
				c = origin
				GOSUB update_square

				#spr = piece_to_color(board(origin))
				board(target) = board(origin)
				board(origin) = 0
				turn = turn XOR 8
				GOSUB move_piece

				'
				' Computer plays
				'
				PRINT AT 238 COLOR 7,":O"

				#depth = #stack_base + STACK_ENTRY * (level + 1)
				#c = USR play_chess

				PRINT AT 238 COLOR 7,":)"

				'
				' Debugging info
				'
				FOR c = 0 TO 9
					d = PEEK(#stack_base + c) / 16
					d = d + $10
					IF d > $19 THEN d = d + 7
					#backtab(160 + c * 2) = d * 8 + 7
					d = PEEK(#stack_base + c) AND 15
					d = d + $10
					IF d > $19 THEN d = d + 7
					#backtab(161 + c * 2) = d * 8 + 7
				NEXT c

				FOR c = 180 TO 199
					#backtab(c) = 0
				NEXT c

				PRINT AT 180 COLOR 7,"SCORE:"
				IF #c < 0 THEN PRINT "-",<>-#c ELSE PRINT " ",<>#c

				PRINT AT 200,"ORIGIN: ",<>best_origin
				PRINT AT 220, "TARGET: ",<>best_target
				PRINT AT 231,"ST: ",<>#stack-#stack_base

				origin = best_origin
				target = best_target

				#spr = piece_to_color(board(origin))
				board(target) = board(origin)
				board(origin) = 0
				turn = turn XOR 8

				GOSUB move_piece
				cursor = target

				GOTO main_loop
			END IF
			c = 0
		END IF

		d = controller_diagonal(c AND $1F)
		IF (d = 8) + (d = 1) + (d = 2) THEN
			IF cursor < 30 THEN
				cursor = cursor + 70
			ELSE
				cursor = cursor - 10
			END IF
		END IF
		IF (d = 4) + (d = 5) + (d = 6) THEN
			IF cursor > 90 THEN
				cursor = cursor - 70
			ELSE
				cursor = cursor + 10
			END IF
		END IF
		IF (d = 2) + (d = 3) + (d = 4) THEN
			IF (cursor % 10) = 8 THEN
				cursor = cursor - 7
			ELSE
				cursor = cursor + 1
			END IF
		END IF
		IF (d = 6) + (d = 7) + (d = 8) THEN
			IF (cursor % 10) = 1 THEN
				cursor = cursor + 7
			ELSE
				cursor = cursor - 1
			END IF
		END IF
		IF d <> 0 THEN debounce = DEBOUNCE_TIME
	END IF

	GOTO main_loop

clear_sprites:	PROCEDURE
	FOR c = 0 TO 7
		SPRITE c, 0
	NEXT c
	END

move_piece:	PROCEDURE
	best_score = 0

	c = ABS(target - origin)

	c = origin
	GOSUB update_square

	c = square_to_screen(origin)
	x = c % 20 * 8
	y = c / 20 * 8
	
	c = square_to_screen(target)
	x2 = c % 20 * 8
	y2 = c / 20 * 8
	
	dx = SGN(x2 - x)
	dy = SGN(y2 - y)

	IF ABS(x - x2) > ABS(y - y2) THEN
		speed = ABS(y - y2) * 64 / ABS(x - x2)
		valid = 0
	ELSE
		speed = ABS(x - x2) * 64 / ABS(y - y2)
		valid = 1
	END IF

	c = 0
	WHILE 1
		SPRITE 0, $0308+x, $0108+y, #spr
		SPRITE 1, $0310+x, $0108+y, #spr+8
		WAIT
		c = c + speed
		IF c >= $40 THEN
			c = c - $40
			IF valid = 0 THEN
				y = y + dy
			ELSE
				x = x + dx
			END IF
		END IF
		IF valid = 0 THEN
			x = x + dx
			IF x = x2 THEN EXIT WHILE
		ELSE
			y = y + dy
			IF y = y2 THEN EXIT WHILE
		END IF
	WEND

	SPRITE 0,0
	SPRITE 1,0

	c = target
	GOSUB update_square
	END

select_square:	PROCEDURE
	selected = c
	GOSUB update_square
	END

deselect_square:	PROCEDURE
	IF selected = 0 THEN RETURN
	c = selected
	selected = 0
	GOSUB update_square
	END

update_board:	PROCEDURE
	FOR c = 21 TO 98
		GOSUB update_square
		IF c % 10 = 8 THEN c = c + 2
	NEXT c
		
	END

update_square:	PROCEDURE
	d = square_to_screen(c)
	IF selected = c THEN
		#c = $2200
	ELSE
		#c = square_to_color(c) 
	END IF
	#c = #c + piece_to_color(board(c) AND $0F)
	#backtab(d) = #c
	IF #c AND $09F8 THEN
		#backtab(d + 1) = #c + 8
	ELSE
		#backtab(d + 1) = #c
	END IF
	END

setup_board:	PROCEDURE
	FOR c = 0 TO 119
		board(c) = 7
	NEXT c

	FOR c = 21 TO 91 STEP 10
		FOR d = 0 TO 7
			board(c + d) = 0
		NEXT d
	NEXT c

	board(91) = $0c
	board(92) = $0a
	board(93) = $0b
	board(94) = $0d
	board(95) = $0e
	board(96) = $0b
	board(97) = $0a
	board(98) = $0c

	FOR d = 81 TO 88
		board(d) = $09
	NEXT d
	FOR d = 31 TO 38
		board(d) = $01
	NEXT d

	board(21) = $04
	board(22) = $02
	board(23) = $03
	board(24) = $05
	board(25) = $06
	board(26) = $03
	board(27) = $02
	board(28) = $04

	END

	'
	' Table for converting disc direction to 8-way direction
	'
controller_diagonal:
	DATA 0, 5, 3, 5, 1, 0, 3, 0
	DATA 7, 7, 0, 0, 1, 0, 0, 0
	DATA 0, 5, 3, 4, 1, 0, 2, 0
	DATA 7, 6, 0, 0, 8, 0, 0, 0

square_to_screen:
	DATA 0,0,0,0,0,0,0,0,0,0
	DATA 0,0,0,0,0,0,0,0,0,0
	DATA 0,0,2,4,6,8,10,12,14,0
	DATA 0,20,22,24,26,28,30,32,34,0
	DATA 0,40,42,44,46,48,50,52,54,0
	DATA 0,60,62,64,66,68,70,72,74,0
	DATA 0,80,82,84,86,88,90,92,94,0
	DATA 0,100,102,104,106,108,110,112,114,0
	DATA 0,120,122,124,126,128,130,132,134,0
	DATA 0,140,142,144,146,148,150,152,154,0
	DATA 0,0,0,0,0,0,0,0,0,0
	DATA 0,0,0,0,0,0,0,0,0,0

square_to_color:
	DATA 0,0,0,0,0,0,0,0,0,0
	DATA 0,0,0,0,0,0,0,0,0,0
	DATA 0,$0400,$0200,$0400,$0200,$0400,$0200,$0400,$0200,0
	DATA 0,$0200,$0400,$0200,$0400,$0200,$0400,$0200,$0400,0
	DATA 0,$0400,$0200,$0400,$0200,$0400,$0200,$0400,$0200,0
	DATA 0,$0200,$0400,$0200,$0400,$0200,$0400,$0200,$0400,0
	DATA 0,$0400,$0200,$0400,$0200,$0400,$0200,$0400,$0200,0
	DATA 0,$0200,$0400,$0200,$0400,$0200,$0400,$0200,$0400,0
	DATA 0,$0400,$0200,$0400,$0200,$0400,$0200,$0400,$0200,0
	DATA 0,$0200,$0400,$0200,$0400,$0200,$0400,$0200,$0400,0
	DATA 0,0,0,0,0,0,0,0,0,0
	DATA 0,0,0,0,0,0,0,0,0,0

piece_to_color:
	DATA $0000,$0800,$0810,$0820,$0830,$0840,$0850,$0000
	DATA $0000,$0807,$0817,$0827,$0837,$0847,$0857,$0000

	ASM INCLUDE "engine.asm"

	INCLUDE "pieces.bas"
