	;
	; Chess engine for Intellivision
	;
	; (c) Copyrigh 2023 Oscar Toledo G.
	; https://nanochess.org/
	;
	; Creation date: Feb/28/2023.
	;

STACK_ENTRY:	EQU 10

piece_total:
	DECLE 3,3,7,3,3,7,7

piece_offsets:
	DECLE displacement+16
	DECLE displacement+20
	DECLE displacement+0
	DECLE displacement+12
	DECLE displacement+8
	DECLE displacement+8
	DECLE displacement+8

displacement:
	DECLE -21,-19,-12,-8,8,12,19,21
	DECLE -10,10,-1,1
	DECLE 9,11,-9,-11
	DECLE -11,-9,-10,-20
	DECLE 9,11,10,20

piece_scores:
	DECLE 0,1,3,3,5,9

PLAY_CHESS:	PROC
	BEGIN

	MVII #-128,R0
	MVO R0,var_BEST_SCORE

	MVI var_TURN,R0
	XORI #8,R0
	MVO R0,var_TURN

	MVII #array_BOARD+21,R3

SR7:	MVI@ R3,R1	; Read square.
	XOR var_TURN,R1	; XOR with current playing side.
	ANDI #$0F,R1	; Remove moved bit.
	DECR R1		; Translate to 0-5.
	CMPI #6,R1	; Is it frontier or empty square?
	BC SR17		; Yes, jump.
	TSTR R1		; Is it a pawn?
	BNE SR8		; No, jump.
	MVI var_TURN,R0
	TSTR R0		; Is it playing black?
	BNE SR25	; No, jump.
SR8:	INCR R1		; Reverse direction for pawn.
SR25:
	ADDI #piece_total,R1
	MVI@ R1,R0
	MVO R0,var_TOTAL_MOVEMENTS
	ADDI #7,R1
	MVI@ R1,R1	; Movements offset in R1.

SR12:	MOVR R3,R2	; Restart target square.

SR9:	ADD@ R1,R2	; Displace in direction.
	MVI@ R3,R4	; Origin square.
	MVI@ R2,R5	; Target square.
	ANDI #$0F,R5	; Empty square?
	BEQ SR10	; Yes, jump.
	;
	; Moving to a square containing something.
	;
	XOR var_TURN,R5
	SUBI #9,R5	; Is it a valid capture?
	CMPI #6,R5
	MVI@ R2,R5
	BC SR18		; No, jump to avoid.
	CMPI #displacement+16,R1	; Moving pawn?
	BNC SR19	; No, jump to capture.
	MVI var_TOTAL_MOVEMENTS,R0
	ANDI #2,R0	; Straight advance?
	BEQ SR18	; Yes, jump to avoid.
	B SR19		; No, jump to capture.

	;
	; Moving to an empty square.
	;
SR10:
	CMPI #displacement+16,R1	; Moving pawn?
	BNC SR19	; No, jump to move.
	MVI var_TOTAL_MOVEMENTS,R0
	SARC R0,2	; Diagonal movement?
	BNOV SR19	; No, jump to move.
	MOVR R3,R0
	DECR R0
	BNC SR29
	INCR R0
	INCR R0
SR29:	CMP var_EN_PASSANT,R0	; Is it a valid en passant?
	BNE SR18		; No, avoid.

	;
	; Save the current state into the simulated stack
	;
SR19:	MVI var_&STACK,R5
	MVO@ R3,R5		; +0 Origin square
	MVO@ R2,R5		; +1 Target square
	MVI@ R3,R0		; Content of origin square.
	MVO@ R0,R5		; +2
	MVI@ R2,R0		; Content of target square.
	MVO@ R0,R5		; +3
	ANDI #7,R0
	CMPI #6,R0		; King eaten?
	BNE SR20		; No, jump.
	MOVR R5,R1
	SUBI #STACK_ENTRY*1+4,R1	
	CMP var_&STACK_BASE,R1	; Is it the first answer to computer move?
	MVII #63,R0		; Yes, maximum plus score.
	BEQ SR26	
	MVII #47,R0		; Yes, maximum score.
SR26:	B SR24

SR20:
	MOVR R0,R4
	ADDI #piece_scores,R4
	MVI@ R4,R0		; Capture score

	MOVR R5,R4
	SUBI #STACK_ENTRY*2+4,R4		; Search depth
	CMP var_&STACK_BASE,R4
	MVI var_BEST_SCORE,R4
	BEQ SR22

	MVO@ R0,R5		; +4

	MVO@ R1,R5		; +5 Pointer to movement table.
	SWAP R1
	MVO@ R1,R5		; +6

	MVI var_EN_PASSANT,R0	; Current en passant state.
	MVO@ R0,R5		; +7
	MVI var_TOTAL_MOVEMENTS,R0	; Total movements remaining.
	MVO@ R0,R5		; +8
	MVI var_BEST_SCORE,R0	; Best score so far.
	MVO@ R0,R5		; +9

	MVO R5,var_&STACK

	CLRR R0			
	MVO R0,var_EN_PASSANT

	MVI@ R3,R4		; R4 = Content of origin square
;	MVI@ R2,R5		; R5 = Content of target square

	CLRR R5
	MVO@ R5,R3		; Clear origin square
	MVO@ R4,R2		; Move to target square

	CALL PLAY_CHESS
	; R0 = Reply score

	MVI var_&STACK,R5
	SUBI #STACK_ENTRY,R5
	MVO R5,var_&STACK

	MVI@ R5,R3	; +0
	ADDI #$0100,R3	; Restore current origin square
	MVI@ R5,R2	; +1
	ADDI #$0100,R2	; Restore current target square
	MVI@ R5,R1	; +2
	MVO@ R1,R3	; Restore content of origin square
	COMR R0		; Negate new score
	MVI@ R5,R1	; +3
	MVO@ R1,R2	; Restore content of target square
	INCR R0
	ADD@ R5,R0	; +4 Subtract from previous score
	MVI@ R5,R1	; +5
	SWAP R1
	ADD@ R5,R1	; +6
	SWAP R1
	MVI@ R5,R4	; +7
	MVO R4,var_EN_PASSANT
	MVI@ R5,R4	; +8
	MVO R4,var_TOTAL_MOVEMENTS

	MVI@ R5,R4	; +9

SR22:
	XORI #$80,R4	; Extend sign
	SUBI #$80,R4
	CMPR R4,R0
	BLT SR23
	BNE SR27
	MVI _rand,R0
	ANDI #$2A,R0
	BNE SR23
	MOVR R4,R0
SR27:
	MOVR R0,R4	; New best score

	MVI var_&STACK,R0
	CMP var_&STACK_BASE,R0
	BNE SR23
	MOVR R3,R0
	SUBI #array_BOARD,R0
	MVO R0,var_BEST_ORIGIN
	MOVR R2,R0
	SUBI #array_BOARD,R0
	MVO R0,var_BEST_TARGET

SR23:
	MVO R4,var_BEST_SCORE

SR18:	MVI@ R3,R0
	ANDI #7,R0	; Was it pawn?
	DECR R0
	BEQ SR11	; Yes, check special.

	CMPI #1,R0	; Knight?
	BEQ SR14	; Yes, end movement.
	CMPI #5,R0	; King?
	BEQ SR14	; Yes, end movement.
	MVI@ R2,R0
	TSTR R0		; To empty square?
	BEQ SR9		; Yes, follow line of squares.
	B SR14

SR11:	MVI@ R1,R0
	CMPI #-10,R0
	BEQ SR15
	CMPI #10,R0
	BNE SR14
SR15:	MVI@ R2,R0
	TSTR R0		; Pawn to empty square?
	BNE SR17	; No, cancel double-square movement.
	MOVR R3,R0
	SUBI #array_BOARD+41,R0
	CMPI #40,R0	; At first top row or bottom row?
	BNC SR17	; No, cancel double-square movement.

SR14:	INCR R1
	MVI var_TOTAL_MOVEMENTS,R4
	DECR R4
	MVO R4,var_TOTAL_MOVEMENTS
	BPL SR12

SR17:	INCR R3
	CMPI #array_BOARD+99,R3
	BNE SR7

	MVI var_BEST_SCORE,R0
	XORI #$80,R0	; Extend sign
	SUBI #$80,R0

SR24:	MVI var_TURN,R1
	XORI #8,R1
	MVO R1,var_TURN

	RETURN
	ENDP

